@@ -61,6 +61,9 @@ static void init_response(struct winbindd_response *response)
 
 /* Close established socket */
 
+#if HAVE_FUNCTION_ATTRIBUTE_DESTRUCTOR
+__attribute__((destructor))
+#endif
 void winbind_close_sock(void)
 {
 	if (winbindd_fd != -1) {
@@ -240,12 +243,6 @@ static int winbind_named_pipe_sock(const char *dir)
 
 		switch (errno) {
 			case EINPROGRESS:
-
-				if (fd < 0 || fd >= FD_SETSIZE) {
-					errno = EBADF;
-					goto error_out;
-				}
-
 				FD_ZERO(&w_fds);
 				FD_SET(fd, &w_fds);
 				tv.tv_sec = CONNECT_TIMEOUT - wait_time;
@@ -390,12 +387,6 @@ int winbind_write_sock(void *buffer, int count, int recursing, int need_priv)
 		struct timeval tv;
 		fd_set r_fds;
 
-		if (winbindd_fd < 0 || winbindd_fd >= FD_SETSIZE) {
-			errno = EBADF;
-			winbind_close_sock();
-			return -1;
-		}
-
 		/* Catch pipe close on other end by checking if a read()
 		   call would not block by calling select(). */
 
@@ -456,11 +447,6 @@ int winbind_read_sock(void *buffer, int count)
 		struct timeval tv;
 		fd_set r_fds;
 
-		if (winbindd_fd < 0 || winbindd_fd >= FD_SETSIZE) {
-			errno = EBADF;
-			winbind_close_sock();
-			return -1;
-		}
 		/* Catch pipe close on other end by checking if a read()
 		   call would not block by calling select(). */
 
