@@ -71,7 +71,6 @@ static int cli_credentials_set_from_ccache(struct cli_credentials *cred,
 	krb5_principal princ;
 	krb5_error_code ret;
 	char *name;
-	char **realm;
 
 	if (cred->ccache_obtained > obtained) {
 		return 0;
@@ -98,8 +97,6 @@ static int cli_credentials_set_from_ccache(struct cli_credentials *cred,
 		return ret;
 	}
 
-	realm = krb5_princ_realm(ccache->smb_krb5_context->krb5_context, princ);
-
 	cli_credentials_set_principal(cred, name, obtained);
 
 	free(name);
