@@ -3,4 +3,4 @@ bool test_SetupCredentials2(struct dcerpc_pipe *p, struct torture_context *tctx,
 			    uint32_t negotiate_flags,
 			    struct cli_credentials *machine_credentials,
 			    int sec_chan_type,
-			    struct creds_CredentialState **creds_out);
+			    struct netlogon_creds_CredentialState **creds_out);
